@@ -53,4 +53,16 @@ class PhbrArgumentParser(NGArgumentParser):
                         dest="output_dir",
                         default="/tmp",
                         help="prediction result output directory (Default=/tmp)",
-                        metavar="OUTPUT_DIR")
\ No newline at end of file
+                        metavar="OUTPUT_DIR")
+        # a flag to indicate whether or not to include the mhci & mhcii result (peptide table)
+        self.parser_preprocess.add_argument("--include-mhci-mhcii-result",
+                                        action="store_true",
+                                        dest="include_mhci_mhcii_result",
+                                        default=False,
+                                        help="flag to indicate whether or not to include mhci & mhcii result (peptide table) in the job description file")
+        # a flag to indicate whether or not to include the mhci & mhcii result (peptide table)
+        self.parser_postprocess.add_argument("--include-mhci-mhcii-result",
+                                action="store_true",
+                                dest="include_mhci_mhcii_result",
+                                default=False,
+                                help="flag to indicate whether or not to include mhci & mhcii result (peptide table) in the post-processed result")
