@@ -821,6 +821,9 @@ def run(parser: Any, **kwargs: Any) -> None:
             f'{result_dir_phbr.parent.resolve()}/results/{curr_job_id}.json'
         ]
     }
+    # add "--include-mhci-mhcii-result" if include_mhci_mhcii_result
+    if kwargs.get('include_mhci_mhcii_result', False):
+        postprocessing_job['shell_cmd'] += ' --include-mhci-mhcii-result'
     job_description.append(postprocessing_job)
 
     for job in job_description:
