@@ -1,6 +1,7 @@
 import argparse
 import json
 import csv
+import edlib
 import os
 import sys
 import shlex
@@ -32,6 +33,16 @@ class TCellClass(str, Enum):
     II = 'ii'
 
 
+def find_mutation_positions(mut_peptide: str, ref_peptide: str) -> List[int]:
+    result = edlib.align(mut_peptide, ref_peptide, mode='NW', task="path")
+    pretty_result = edlib.getNiceAlignment(result, mut_peptide, ref_peptide)
+    alignment = pretty_result['matched_aligned']
+    dot_indices = [i for i, char in enumerate(alignment) if char == '.']
+    return ','.join(str(i+1) for i in dot_indices)
+
+def add_mutation_position_column(df: pd.DataFrame) -> pd.DataFrame:
+    df['mutation_position'] = df.apply(lambda row: find_mutation_positions(row['mut_peptide'], row['ref_peptide']), axis=1)
+    return df
 
 def detect_file_type(filepath: str) -> str:
     try:
@@ -195,6 +206,25 @@ def remove_empty_rank_rows(mhc_pred_file: str) -> None:
     df.to_csv(mhc_pred_file, sep='\t', index=False)
 
 
+def add_sequence_number_to_phbr_output(phbr_output_file: str, sequence_to_seqnum_dict: Dict[str, int]) -> None:
+    df = pd.read_csv(phbr_output_file, sep='\t')
+    print(df.head())
+    print('************************************************')
+    print('sequence_to_seqnum_dict:')
+    for seq, num in sequence_to_seqnum_dict.items():
+        print(f'  {seq}: {num}')
+    print('************************************************')
+    # sys.exit()
+    df['seq #'] = df['peptide'].map(sequence_to_seqnum_dict)
+    # Reorder columns to put 'seq #' first
+    cols = ['seq #'] + [col for col in df.columns if col != 'seq #']
+    df = df[cols]
+    print(df.head())
+    print('************************************************')
+    # Save the updated dataframe back to the original file
+    df.to_csv(phbr_output_file, sep='\t', index=False)
+
+
 def mhci_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
     """
     Process MHCI data and convert it to PHBR format.
@@ -247,6 +277,16 @@ def mhci_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
         peptide_table_path = mhc_binding_result_json2tsv(Path(data['mhc_peptide_json']), TCellClass.I, is_aggregated_result=True)
         sequence_table_path = data['mhc_sequence_tsv']
 
+    # Capture the sequence number
+    # NOTE: Currently, it seems that PHBR will ignore duplicate sequences.
+    sequence_table_df = pd.read_csv(sequence_table_path, sep='\t')
+
+    if mut_pos_col or mut_pos:
+        validators.validate_mutation_position(sequence_table_df, mut_pos_col, mut_pos)
+    # Create dictionary mapping sequence to sequence number
+    sequence_to_seqnum_dict = dict(zip(sequence_table_df['sequence'], sequence_table_df['seq #']))
+    
+
 
     '''
     CASE 2: MHCI binding prediction needs to be run.
@@ -272,12 +312,6 @@ def mhci_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
     tmp_mhc_pred_file = tempfile.NamedTemporaryFile(prefix="phbr-input-", delete=False, dir=output_dir)
     mhc2phbr_fpath = parser.find_file_path(start_dir=root_path, filename='mhc2phbr.py')
 
-    # print('--------------------------------------------------')
-    # print('tmp_mhc_pred_file: ', tmp_mhc_pred_file)
-    # print('tmp_mhc_pred_file.name: ', tmp_mhc_pred_file.name)
-    # print('--------------------------------------------------')
-    # sys.exit()
-
     # Specify --rank-colname
     # NOTE: if 'netmhcpan percentile' is not found, then use 'percentile' 
     #       as the default rank_colname
@@ -298,10 +332,10 @@ def mhci_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
     ] + mhc_args
 
     # NOTE: If neither is specified, it will use the central position
-    if mut_pos_col:
-        mhc_args = mhc_args + ['--sequence-mutation-position-colname', mut_pos_col]
-    elif mut_pos:
+    if mut_pos:
         mhc_args = mhc_args + ['--mutation-position', mut_pos,]
+    elif mut_pos_col:
+        mhc_args = mhc_args + ['--sequence-mutation-position-colname', mut_pos_col]
 
     command = ['python', mhc2phbr_fpath] + mhc_args
 
@@ -320,7 +354,6 @@ def mhci_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
         print('Return Code:', e.returncode)
 
 
-
     '''
     Use PHBR output from 'mhc2phbr.py' (previous step) as input to PHBR.
     '''
@@ -348,6 +381,8 @@ def mhci_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
         print("Running command:", command)
         print('--------------------------------------------------')
         subprocess.run(command, capture_output=True, text=True, check=True)
+        # Add sequence number to the PHBR output
+        add_sequence_number_to_phbr_output(output_file, sequence_to_seqnum_dict)
         print(f'Final result of saved to {output_file}')
     except subprocess.CalledProcessError as e:
         print('Error:', e.stderr)
@@ -412,6 +447,16 @@ def mhcii_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
         peptide_table_path = mhc_binding_result_json2tsv(Path(data['mhc_peptide_json']), TCellClass.II, is_aggregated_result=True)
         sequence_table_path = data['mhc_sequence_tsv']
 
+    # Capture the sequence number
+    # NOTE: Currently, it seems that PHBR will ignore duplicate sequences.
+    sequence_table_df = pd.read_csv(sequence_table_path, sep='\t')
+
+    if mut_pos_col or mut_pos:
+        validators.validate_mutation_position(sequence_table_df, mut_pos_col, mut_pos)
+
+    # Create dictionary mapping sequence to sequence number
+    sequence_to_seqnum_dict = dict(zip(sequence_table_df['sequence'], sequence_table_df['seq #']))
+
 
     '''
     CASE 2: MHCII binding prediction needs to be run.
@@ -456,10 +501,10 @@ def mhcii_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
     ] + mhc_args
 
     # NOTE: If neither is specified, it will use the central position
-    if mut_pos_col:
-        mhc_args = mhc_args + ['--sequence-mutation-position-colname', mut_pos_col]
-    elif mut_pos:
+    if mut_pos:
         mhc_args = mhc_args + ['--mutation-position', mut_pos,]
+    elif mut_pos_col:
+        mhc_args = mhc_args + ['--sequence-mutation-position-colname', mut_pos_col]
 
     command = ['python', mhc2phbr_fpath] + mhc_args
 
@@ -505,6 +550,8 @@ def mhcii_to_phbr(data: Dict[str, Any], parser: PhbrArgumentParser) -> None:
         print("Running command:", command)
         print('--------------------------------------------------')
         subprocess.run(command, capture_output=True, text=True, check=True)
+        # Add sequence number to the PHBR output
+        add_sequence_number_to_phbr_output(output_file, sequence_to_seqnum_dict)
         print(f'Final result of saved to {output_file}')
     except subprocess.CalledProcessError as e:
         print('Error:', e.stderr)
@@ -546,6 +593,7 @@ def combine_mhc_results(mhci_output: str, mhcii_output: str, output_file: str) -
     
     # Define the desired column order
     column_order = [
+        'seq #',  # Sequence number column
         'peptide',
         '#A', '#B', '#C',  # MHCI columns
         '#DP', '#DQ', '#DR',  # MHCII columns
@@ -581,6 +629,33 @@ def replace_extension_with_json(filepath: str) -> str:
     return str(Path(filepath).with_suffix('.json'))
 
 
+def determine_mutation_position(df: pd.DataFrame) -> str:
+    """
+    Determine the mutation position from the sequence column.
+    
+    Args:
+        df: DataFrame containing sequence information
+        
+    Returns:
+        Column name containing mutation position, or None if not found
+    """
+    # Get all column names
+    columns = df.columns.tolist()
+    
+    # Convert column names to lowercase for case-insensitive matching
+    columns_lower = [col.lower() for col in columns]
+    
+    # Look for variations of mutation position column names
+    for i, col in enumerate(columns_lower):
+        # Check if column contains both 'mut' and 'pos' in any order
+        if ('mut' in col and 'pos' in col) or ('pos' in col and 'mut' in col):
+            return columns[i]  # Return original column name
+            
+    return None
+
+
+
+
 def main() -> None:
     parser = PhbrArgumentParser()
     args = parser.parse_args()
@@ -624,18 +699,68 @@ def main() -> None:
             'output_dir': tmp_output_dir,
         }
 
+        # print("************************************************")
+        # print(data)
+        # print(args)
+        # print("************************************************")
+        # mhc_sequence_tsv = data['mhc_sequence_tsv']
+        # print(f"mhc_sequence_tsv: {mhc_sequence_tsv}")
+        # sys.exit()
+
+        # data['mutation_position'] = args.mut_pos
+        data['mutation_position'] = data.get('mutation_position', None)
+
+
+        if 'mhc_sequence_tsv' in data:
+            df = pd.read_csv(data['mhc_sequence_tsv'], sep='\t')
+            is_paired_peptide_table = validators.validate_paired_peptides_table(df)
+            print(f"is_paired_peptide_table: {is_paired_peptide_table}")
+
+            if is_paired_peptide_table:                
+                df = add_mutation_position_column(df)
+
+                # Remove the reference peptide column which is the second column
+                df = df.drop(columns=df.columns[1])
+
+                # Rename the first column to 'sequence'
+                df = df.rename(columns={df.columns[0]: 'sequence'})
+                
+                df.insert(0, 'seq #', range(1, len(df) + 1))
+
+
         # NOTE: Turn "input_neoepitopes" into "mhc_sequence_tsv"
         if 'input_neoepitopes' in data:
             df = pd.read_csv(StringIO(data['input_neoepitopes']), sep='\t')
-            df = df.rename(columns=lambda x: 'sequence' if 'peptide' in x.lower() else x)
-            df.insert(0, 'seq #', range(1, len(df) + 1))
-
-            with tempfile.NamedTemporaryFile(mode='w+', suffix='.tsv', delete=False, dir=tmp_output_dir) as tmp_file:
-                df.to_csv(tmp_file.name, index=False, sep='\t')
-                print(f"Temporary file created at: {tmp_file.name}")
+            is_paired_peptide_table = validators.validate_paired_peptides_table(df)
+            print(f"is_paired_peptide_table: {is_paired_peptide_table}")
             
-            data['mhc_sequence_tsv'] = tmp_file.name
+            if is_paired_peptide_table:
+                df = add_mutation_position_column(df)
 
+                # Remove the reference peptide column which is the second column
+                df = df.drop(columns=df.columns[1])
+                
+                # Rename the first column to 'sequence'
+                df = df.rename(columns={df.columns[0]: 'sequence'})
+                
+                df.insert(0, 'seq #', range(1, len(df) + 1))
+
+            else: 
+                df = df.rename(columns=lambda x: 'sequence' if 'peptide' in x.lower() else x)
+                df.insert(0, 'seq #', range(1, len(df) + 1))
+
+
+        mut_pos_col = determine_mutation_position(df)
+        # print(f"Mutation position column: {mut_pos_col}")
+
+        if mut_pos_col:
+            data['mutation_position_colname'] = mut_pos_col
+
+        with tempfile.NamedTemporaryFile(mode='w+', suffix='.tsv', delete=False, dir=tmp_output_dir) as tmp_file:
+            df.to_csv(tmp_file.name, index=False, sep='\t')
+            print(f"Temporary file created at: {tmp_file.name}")
+        
+        data['mhc_sequence_tsv'] = tmp_file.name
 
         # ================================
         # Handle combined case
